/*
 * game.cpp
 *
 *  Created on: 28/mar/2012
 *      Author: bertini
 */


#include "Weapon.h"
#include "Character.h"
#include "Dungeon.h"
#include "Knight.h"
#include "Monster.h"
#include "MeleeWeapon.h"
#include "MissileWeapon.h"
#include "Fang.h"
#include "MovEnvElementAdapter.h"
#include "EnvironmentElementAdapter.h"
#include "Chest.h"

#include <vector>
#include <map>
#include <iostream>
#include <memory>

using namespace std;

int main() {

	vector<Character*> allCharacters;
	multimap<int, Character*> mapCharacters;

	unique_ptr<Character> hero( new Knight( 15, 18 ) ); // requires C++11 compiler or Boost
														// if not available then use a Character*

	Weapon* sword = 0;
	sword = new MeleeWeapon;
	sword->setStrenght( 15 );
	hero->setWeapon( sword );
	//allCharacters.push_back( hero );
	// in multimap there's no operator[] (unlike in map) - use the insert method to insert a pair composed of key and value
	//mapCharacters.insert( pair<int, Character*>(hero->getHP(), hero) );

	Character* hero2 = 0;
	hero2 = new Knight( static_cast<const Knight&>(*hero) );
	allCharacters.push_back( hero2 );
	// in multimap there's no operator[] (unlike in map) - use the insert method to insert a pair composed of key and value
	mapCharacters.insert( pair<int, Character*>(hero2->getHP(), hero2) );

	const int maxMonsters = 4;
	Character **monsters;
	monsters = new Character*[4];
	for(int i=0; i< 3; i++)
		monsters[i] = new Monster;
	monsters[3] = new Knight; // evil knight

	Weapon* fang = 0;
	fang = new MeleeWeapon( 10 );

	for(int i = 0; i < maxMonsters; i++) {
		monsters[i]->setHP(14);
		monsters[i]->setWeapon( fang );
		allCharacters.push_back( monsters[i] );
		// in multimap there's no operator[] (unlike in map) - use the insert method to insert a pair composed of key and value
		mapCharacters.insert( pair<int, Character*>(monsters[i]->getHP(), monsters[i]) );
	}
	MovableEnvironmentElement  door(1, 1, 3, true);
	Character* adaptedDoor = new MovEnvElementAdapter(door);
	allCharacters.push_back( adaptedDoor );
	hero->fight( *(allCharacters[maxMonsters+1]) );
	if( adaptedDoor->getHP() <= 0 )
		cout << "Broken door!" << endl;
	MovableEnvironmentElement* door2 = new MovableEnvironmentElement(1, 1, 4, true); // the Adapter needs a base class
	Character* doorAdapter = new EnvironmentElementAdapter( door2 );
	allCharacters.push_back( doorAdapter );
	hero->fight( *(allCharacters[maxMonsters+2]) );
	if( doorAdapter->getHP() <= 0 )
		cout << "Second door is broken !" << endl;
	MovableEnvironmentElement* chest = new Chest(1, 1, 100, 4, true); // the Adapter needs a base class
	Character* chestAdapter = new EnvironmentElementAdapter( chest );
	allCharacters.push_back( chestAdapter );
	hero->fight( *(allCharacters[maxMonsters+3]) );
	if( chestAdapter->getHP() <= 0 ) {
		cout << "Forziere rotto !" << endl;
		Chest* chestP = dynamic_cast<Chest*>(chest); // to get the treasure we need the derived class
		if( chestP )
			cout << "- Coin: " << chestP->getTreasure() << endl;
	}

	vector<Character*>::const_iterator itrCharacters;
	for( itrCharacters = allCharacters.begin(); itrCharacters != allCharacters.end(); itrCharacters++ ) {
		cout << "HP Character: " << (*itrCharacters)->getHP() << endl;
		Knight* aK;
		if( (aK = dynamic_cast<Knight*>(*itrCharacters)) )
			cout << "Ability: " << aK->getAbility() << endl;
	}
	map<int, Character*>::iterator mapItr;
	mapItr = mapCharacters.find( 14 );
	(*mapItr).second->setHP( 15 );

	int monsterIdx = 0;
	cout << "Numero mostro da attaccare (da 0 a " << (maxMonsters-1) << ")" << endl;
	do {
		cin >> monsterIdx;
	} while( ((monsterIdx>=maxMonsters) || (monsterIdx<0)) );
	const int maxAttacks = 3;
	for(int i = 0; i < maxAttacks; i++) {
		cout << "Attacco eroe... ";
		hero->fight( *monsters[monsterIdx] );
		if( monsters[monsterIdx]->getHP() <= 0 ) {
			cout << "Il mostro  morto !" << endl;
			break;
		}
		Monster* coward = dynamic_cast<Monster*>(monsters[monsterIdx]);
		if( coward )
			coward->flee();
		cout << "Attacco mostro... ";
		monsters[monsterIdx]->fight( *hero );
		if( hero->getHP() <= 0 ) {
			cout << "L'eroe  morto !" << endl;
			break;
		}
	}

	//delete hero;
	delete sword;
	for(int i=0; i<4; i++)
		delete monsters[i];
	delete[] monsters;
	delete fang;
}

